import { Logger } from '@jdbernard/logging';

export interface EditableItemState<T> {
  value: T;
  editing?: boolean;
  saving: boolean;
  error: boolean;
}

export function asOptional<T>(
  fun: () => T,
  logger: Logger | null = null
): T | null {
  try {
    return fun();
  } catch (err: any) {
    logger?.warn(err);
    return null;
  }
}

export function batch<T>(arr: T[], batchSize: number): T[][] {
  const result: T[][] = [];
  let curBatch: T[] = [];

  for (let i = 0; i < arr.length; i++) {
    if (i > 0 && i % batchSize === 0) {
      result.push(curBatch);
      curBatch = [];
    }

    curBatch.push(arr[i]);
  }

  result.push(curBatch);
  return result;
}

export function clone<T>(obj: T): T {
  return JSON.parse(JSON.stringify(obj));
}

export function notNullOrUndef<T>(val: T | undefined | null): val is T {
  return val !== undefined && val !== null;
}

export function newLoggedError(logger: Logger, msg: string): Error {
  const err = new Error(msg);
  logger.error(msg);
  return err;
}

export function requireDefined<T>(
  logger: Logger,
  name: string,
  val: T | null | undefined
): T {
  if (val === null || val === undefined) {
    throw newLoggedError(
      logger,
      `${name} must be defined and non-null but is not`
    );
  }
  return val;
}

/* I wish I could write this, but don't know how to make TypeScript smart
 * enough to infer types correctly against keys. I'ld like, for example, to be
 * able to use this as:
 *
 *     const { api, auth } = requireDefined(logger, {
 *       api: possiblyGetApi(),
 *       auth: possiblyGetAuth()
 *     });
 *
 * But as written, it will infer the types of both api and auth as
 * AuthService | LiveBudgetApiClient
 */
/*
export function requireDefined<T>(logger: Logger, thingsThatMustBeDefined: Record<string, T | null>): Record<string, T> {
  const retVal: Record<string, T> = {};
  Object.entries(thingsThatMustBeDefined).map(([k, v]) => {
    if (v === null || v === undefined) {
      throw newLoggedError(
        logger,
        `${k} must be defined and non-null but is not`
      );
    } else {
      retVal[k] = v;
    }
  });
  return retVal;
}
*/
